@@ -8,22 +8,28 @@ import (
 	"path/filepath"
 	"reflect"
 	"sort"
+	"strconv"
 	"strings"
+	"time"
 
-	"github.com/pkg/errors"
-	"sigs.k8s.io/controller-runtime/pkg/client"
-
+	yaml "github.com/ghodss/yaml"
 	operv1 "github.com/openshift/api/operator/v1"
 	"github.com/openshift/cluster-network-operator/pkg/bootstrap"
 	"github.com/openshift/cluster-network-operator/pkg/render"
+	"github.com/pkg/errors"
 	corev1 "k8s.io/api/core/v1"
 	uns "k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
+	types "k8s.io/apimachinery/pkg/types"
+	"k8s.io/apimachinery/pkg/util/wait"
+	"sigs.k8s.io/controller-runtime/pkg/client"
 )
 
 const OVN_NB_PORT = "9641"
 const OVN_SB_PORT = "9642"
 const OVN_NB_RAFT_PORT = "9643"
 const OVN_SB_RAFT_PORT = "9644"
+const CLUSTER_CONFIG_NAME = "cluster-config-v1"
+const CLUSTER_CONFIG_NAMESPACE = "kube-system"
 
 // renderOVNKubernetes returns the manifests for the ovn-kubernetes.
 // This creates
@@ -52,10 +58,11 @@ func renderOVNKubernetes(conf *operv1.NetworkSpec, bootstrapResult *bootstrap.Bo
 	data.Data["OVN_SB_PORT"] = OVN_SB_PORT
 	data.Data["OVN_NB_RAFT_PORT"] = OVN_NB_RAFT_PORT
 	data.Data["OVN_SB_RAFT_PORT"] = OVN_SB_RAFT_PORT
+	data.Data["OVN_NB_RAFT_ELECTION_TIMER"] = os.Getenv("OVN_NB_RAFT_ELECTION_TIMER")
+	data.Data["OVN_SB_RAFT_ELECTION_TIMER"] = os.Getenv("OVN_SB_RAFT_ELECTION_TIMER")
+	data.Data["OVN_CONTROLLER_INACTIVITY_PROBE"] = os.Getenv("OVN_CONTROLLER_INACTIVITY_PROBE")
 	data.Data["OVN_NB_DB_LIST"] = dbList(bootstrapResult.OVN.MasterIPs, OVN_NB_PORT)
 	data.Data["OVN_SB_DB_LIST"] = dbList(bootstrapResult.OVN.MasterIPs, OVN_SB_PORT)
-	data.Data["OVN_NB_ADDR_LIST"] = addrList(bootstrapResult.OVN.MasterIPs, OVN_NB_PORT)
-	data.Data["OVN_SB_ADDR_LIST"] = addrList(bootstrapResult.OVN.MasterIPs, OVN_SB_PORT)
 	data.Data["OVN_MASTER_IP"] = bootstrapResult.OVN.MasterIPs[0]
 	data.Data["OVN_MIN_AVAILABLE"] = len(bootstrapResult.OVN.MasterIPs)/2 + 1
 	data.Data["LISTEN_DUAL_STACK"] = listenDualStack(bootstrapResult.OVN.MasterIPs[0])
@@ -137,9 +144,12 @@ func isOVNKubernetesChangeSafe(prev, next *operv1.NetworkSpec) []error {
 	if !reflect.DeepEqual(pn.GenevePort, nn.GenevePort) {
 		errs = append(errs, errors.Errorf("cannot change ovn-kubernetes genevePort"))
 	}
+	if pn.HybridOverlayConfig == nil && nn.HybridOverlayConfig != nil {
+		errs = append(errs, errors.Errorf("cannot start a hybrid overlay network after install time"))
+	}
 	if pn.HybridOverlayConfig != nil {
 		if !reflect.DeepEqual(pn.HybridOverlayConfig, nn.HybridOverlayConfig) {
-			errs = append(errs, errors.Errorf("once set cannot change ovn-kubernetes Hybrid Overlay Config"))
+			errs = append(errs, errors.Errorf("cannot edit a running hybrid overlay network"))
 		}
 	}
 
@@ -147,6 +157,7 @@ func isOVNKubernetesChangeSafe(prev, next *operv1.NetworkSpec) []error {
 }
 
 func fillOVNKubernetesDefaults(conf, previous *operv1.NetworkSpec, hostMTU int) {
+
 	if conf.DefaultNetwork.OVNKubernetesConfig == nil {
 		conf.DefaultNetwork.OVNKubernetesConfig = &operv1.OVNKubernetesConfig{}
 	}
@@ -174,15 +185,37 @@ func networkPluginName() string {
 	return "ovn-kubernetes"
 }
 
+type replicaCountDecoder struct {
+	ControlPlane struct {
+		Replicas string `json:"replicas"`
+	} `json:"controlPlane"`
+}
+
 func boostrapOVN(kubeClient client.Client) (*bootstrap.BootstrapResult, error) {
+	clusterConfig := &corev1.ConfigMap{}
+	clusterConfigLookup := types.NamespacedName{Name: CLUSTER_CONFIG_NAME, Namespace: CLUSTER_CONFIG_NAMESPACE}
 	masterNodeList := &corev1.NodeList{}
-	matchingLabels := &client.MatchingLabels{"node-role.kubernetes.io/master": ""}
-	if err := kubeClient.List(context.TODO(), masterNodeList, matchingLabels); err != nil {
-		return nil, err
+
+	if err := kubeClient.Get(context.TODO(), clusterConfigLookup, clusterConfig); err != nil {
+		return nil, fmt.Errorf("Unable to bootstrap OVN, unable to retrieve cluster config: %s", err)
 	}
 
-	if len(masterNodeList.Items) == 0 {
-		return nil, fmt.Errorf("unable to bootstrap OVN, no master nodes found")
+	rcD := replicaCountDecoder{}
+	if err := yaml.Unmarshal([]byte(clusterConfig.Data["install-config"]), &rcD); err != nil {
+		return nil, fmt.Errorf("Unable to bootstrap OVN, unable to unmarshal install-config: %s", err)
+	}
+
+	controlPlaneReplicaCount, _ := strconv.Atoi(rcD.ControlPlane.Replicas)
+
+	err := wait.PollImmediate(2*time.Second, 60*time.Second, func() (bool, error) {
+		matchingLabels := &client.MatchingLabels{"node-role.kubernetes.io/master": ""}
+		if err := kubeClient.List(context.TODO(), masterNodeList, matchingLabels); err != nil {
+			return false, err
+		}
+		return len(masterNodeList.Items) != 0 && controlPlaneReplicaCount == len(masterNodeList.Items), nil
+	})
+	if err != nil {
+		return nil, fmt.Errorf("Unable to bootstrap OVN, expected amount of control plane nodes (%v) do not match found (%v): %s", controlPlaneReplicaCount, len(masterNodeList.Items), err)
 	}
 
 	ovnMasterIPs := make([]string, len(masterNodeList.Items))
@@ -218,14 +251,6 @@ func dbList(masterIPs []string, port string) string {
 	return strings.Join(addrs, ",")
 }
 
-func addrList(masterIPs []string, port string) string {
-	addrs := make([]string, len(masterIPs))
-	for i, ip := range masterIPs {
-		addrs[i] = "ssl://" + net.JoinHostPort(ip, port)
-	}
-	return strings.Join(addrs, ",")
-}
-
 func listenDualStack(masterIP string) string {
 	if strings.Contains(masterIP, ":") {
 		// IPv6 master, make the databases listen dual-stack
